const tt = require('./tokentype');

/**
 * @param {object} axis 
 * @param {boolean} isFirst 
 */
function axisToStr(axis, isFirst) {
	if (isFirst) {
		return '';
	}

	if (axis === tt.doubleSlash) {
		// child or descendent
		return ' ';
	} else if (axis === tt.slash) {
		return ' > ';
	}

	throw Error('!Unexpected axis');
}
/**
 * @param {object} node
 * @return {string}
 */
function nodeTestToStr(node) {
	if (node.value === '*') {
		return '';
	}

	return node.value;
}
/**
 * @param {object} node
 */
function attrFilterToStr(node) {
	const attr = node.attr;

	if (attr === 'id') {
		return '#' + node.value;
	} else if (attr === 'class') {
		return '.' + node.value;
	} else {
		return '[' + attr + '="' + node.value + '"]';
	}
}
/**
 * @param {object} node
 */
function containsToStr(node) {
	if (node.name !== 'contains') {
		throw new Error('Unsupported function: ' + node.name);
	}

	if (node.arg.length !== 2) {
		throw new Error('Expect contains function to have 2 arguments');
	}

	if (node.arg[0].name !== 'text') {
		throw new Error('Only text function is supported inside contains');
	}

	return ':contains(' + node.arg[1].name + ')';
}
/**
 * @param {object} node
 */
function predicateToStr(node) {
	switch (node.type) {
		case 'positionFilter':
			return ':nth-child(' + node.position + ')';
		case 'attributeFilter':
			return attrFilterToStr(node);
		case 'function':
			return containsToStr(node);
		case 'name':
			return ':has(' + node.name + ')';

		default:
			throw new Error('!Unhandled predicate');
	}
}
/**
 * @param {{axis: object, nodeTest: object, predicate: object[]}} step
 * @param {number} index
 */
function stepToSel(step, index) {
	const axis = axisToStr(step.axis, index === 0);
	const nodeTest = nodeTestToStr(step.nodeTest);
	const predicate = step.predicate.map(predicateToStr);

	return axis + nodeTest + predicate.join('');
}
/**
 * @param {object[]} steps
 * @return {string}
 */
function xpath2css(steps) {
	const selectorList = steps.map(stepToSel);

	return selectorList.join('');
}

module.exports = xpath2css;
